#! /usr/software/bin/perl5.14.0

# $Id: //depot/tools/staf/main/start_STAF.pl#47 $
# Author: dl-nate-dev@netapp.com

#
# Copyright (c) 2007-2013 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

use strict;
use warnings;
use Getopt::Long;
use Config;
use File::Spec::Functions qw(catdir);
use File::Path qw(mkpath);
use lib '/usr/software/test/share/nate/last/lib';
# If it's not found (pNATE), it's ok

my $staf_dir             = ($^O =~ /mswin/i) ? "X:\\staf\\current" : '/usr/software/test/staf/current';
my $nate_dir             = ($^O =~ /mswin/i) ? 'X:\nate\last' : '/usr/software/test/share/nate/last';
my $ipv6                 = 1;
my $force                = 0;
my $install_path         = undef;
my $update               = 1;
my $startup_path         = undef;
my $start                = 1;
my $unix_user            = 'root';
my $staf_datadir_symlink = undef;
my $shell                = undef;
my $minwin_drive         = 'x';
my $minwin_unc           = '';
my $minwin_user          = '';
my $minwin_passwd        = '';
my $map                  = 0;
my $autologon_disable    = 0;
my $autologon_domain     = '';
my $autologon_user       = '';
my $autologon_passwd     = '';
my $autologon_regpath    = 'HKLM\Software\Microsoft\Windows NT\CurrentVersion\Winlogon';
my $dir_cache_regpath    = 'HKLM\System\CurrentControlSet\Services\Lanmanworkstation\Parameters';
my $start_mysqld         = 1;

my $Already_Started = "STAFProc is already Running.\n".
                      "Contact dl-nate-dev\@netapp.com if you think this is a problem.\n";
my @argv = @ARGV;
my $dashes = '--';
if ($^O =~ /mswin/i) {
    $dashes = '-';
}

sub usage () {
    return
        "\nusage: $0 [${dashes}path <staf dir>] [${dashes}[no]ipv6]\n"
      . "\t[${dashes}install <local path>] [${dashes}[no]update]\n"
      . "\t[${dashes}startup_path <local path>] [${dashes}unix_user <uesrid>]\n"
      . "\t[${dashes}staf_datadir_symlink </path>] [${dashes}shell </path/to/bash>]\n"
      . "\t[${dashes}minwin_drive <x>] [${dashes}minwin_unc <\\server\\share>]\n"
      . "\t[${dashes}minwin_user <user>] [${dashes}minwin_passwd <password>]\n"
      . "\t[${dashes}map] [${dashes}autologon_domain <short domain name>]\n"
      . "\t[${dashes}autologon_user <user>] [${dashes}autologon_passwd <password>]\n"
      . "\t[${dashes}autologon_disable] [${dashes}[no]mysqld]\n\n"
      . "\tdefaults:\n"
      . "\t${dashes}path $staf_dir \n"
      . "\t${dashes}noipv6\n"
      . "\t${dashes}mysqld\n"
      . "\t${dashes}unix_user $unix_user (only meaningful on unix machines)\n"
      . "\t${dashes}staf_datadir_symlink undef (only meaningful on unix machines)\n\n";
}

GetOptions("staf_datadir_symlink=s" => \$staf_datadir_symlink,
           'path=s'                 => \$staf_dir,
           'natedir=s'              => \$nate_dir,
           "ipv6!"                  => \$ipv6,
           "start!"                 => \$start,
           "install=s"              => \$install_path,
           "startup_path=s"         => \$startup_path,
           "update!"                => \$update,
           "force!"                 => \$force,
           "unix_user=s"            => \$unix_user,
           "shell=s"                => \$shell,
           "minwin_drive=s"         => \$minwin_drive,
           "minwin_unc=s"           => \$minwin_unc,
           "minwin_user=s"          => \$minwin_user,
           "minwin_passwd=s"        => \$minwin_passwd,
           "autologon_domain=s"     => \$autologon_domain,
           "autologon_user=s"       => \$autologon_user,
           "autologon_passwd=s"     => \$autologon_passwd,
           "autologon_disable!"     => \$autologon_disable,
           "map!"                   => \$map,
           "mysqld!"                => \$start_mysqld,
           ) or die usage();
die usage() if (@ARGV);
push(@INC,$nate_dir);

# If STAF is run as 'root' don't start mysql server as it will be start by 'ntest'.
# but if STAF is run as 'nobody' then 'ntest' won't have enough privileges to start
# mysql server, so start it
$start_mysqld = 0 if $unix_user eq 'root';

eval{
    require Tharn_UsageService;
    my %toolargs = (tool              => 'staf',
                    action            => $staf_dir,
                    osname            => $^O,
                    natedir           => $nate_dir,
                    ipv6              => $ipv6,
                    start             => $start,
                    install           => $install_path,
                    startup_path      => $startup_path,
                    force             => $force,
                    unix_user         => $unix_user,
                    shell             => $shell,
                    minwin_drive      => $minwin_drive,
                    minwin_unc        => $minwin_unc,
                    minwin_user       => $minwin_user,
                    minwin_passwd     => $minwin_passwd,
                    autologon_domain  => $autologon_domain,
                    autologon_user    => $autologon_user,
                    autologon_passwd  => $autologon_passwd,
                    autologon_disable => $autologon_disable,
                    mysqld            => $start_mysqld,
                    map               => $map);
    foreach my $k (keys(%toolargs))
    {
        delete($toolargs{$k}) if(!defined($toolargs{$k}));
    }
    # We may not have imported it (if this is pNATE).
    Tharn_UsageService::log_usage(params => \%toolargs, elmo => 1);
};

@argv = grep {$_ =~ s/^-{1,2}force$|^-{1,2}(?:|no)start$//;$_} @argv;

my $VERSION = '3.0';

eval{main();};
my $error = $@;
die $error if($error);

sub main {
    my $service = 'netapp_staf';
    my %subst = (PERL         => $^X,
                 START_STAF   => $0,
                 VERSION      => $VERSION,
                 UPDATE       => $update,
                 INSTALL_FROM => $staf_dir,
                 STARTUP_ARGS => join(' ', (@argv, '--nostart')),
                 STAFDIR      => $staf_dir,
                 MYSQLD       => $start_mysqld,
                 NATEDIR      => $nate_dir,);

    my $fresh_install = 0;
    if($install_path)
    {
        #make $install is if doesn't exist
        my $full_install_path = catdir($install_path, 'STAF');
        if(! -e $full_install_path)
        {
            mkpath($full_install_path);
            $fresh_install = 1;
        }
        else
        {
            my $diff = 'diff';
            my $null = '/dev/null';
            if ($^O =~ /mswin/i)
            {
                $diff = 'FC /L';
                $null = 'NUL';
            }
            foreach my $path ('*',catdir('bin','*'),catdir('lib','*'),catdir('codepage','*'),catdir('include','*'))
            {
                my $cmd = $diff.' '.catdir($staf_dir,$path).' '.catdir($full_install_path, $path).' > '.$null.' 2>&1';
                qx($cmd);
                if($?) {
                    $fresh_install = 1;
                }
                last if($fresh_install);
            }
        }

        die "\n$install_path must be a directory\n" if(! -d $install_path);

        $subst{INSTALL_FROM} = $staf_dir;
        $staf_dir = $full_install_path;
        $subst{STAFDIR} = $staf_dir;
    }
    else
    {
        $subst{UPDATE} = 0;
    }

    if ($^O =~ /mswin/i) # Windows
    {
        chdir("C:\\");
        require Win32;
        my $win_user = Win32::LoginName();
        no strict 'subs';
        my $user_startup_path = Win32::GetFolderPath(&Win32::CSIDL_STARTUP);
        if($user_startup_path)
        {
            unlink(catdir($user_startup_path,"$service.bat"));
        }
        my $common_startup_dir = Win32::GetFolderPath(&Win32::CSIDL_COMMON_STARTUP);
        my $startup = $startup_path || $common_startup_dir;
        unlink(catdir($common_startup_dir, "$service.bat")) if($startup ne $common_startup_dir);
        if(!-e $startup)
        {
            mkpath($startup) or die "Could Not create $startup: $!\n";
        }
        use strict 'subs';

        die "Could not find directory:\"$startup\"\n" if ( ! -d $startup );

        # Perl on windows Lies about it's location so we have to do this hack
        chomp(my $history = `DOSKEY /HISTORY`);
        my @history_list = split("\n",$history);
        @history_list = reverse(@history_list);
        my $cmd;
        foreach my $command(@history_list)
        {
            if($command ne "DOSKEY /HISTORY")
            {
                $cmd = $command;
                last;
            }
        }

        if($cmd)
        {
            # the following regex should make this:
            #   "    \\server\so   me\pl-ace\perl\bin\perl -d -w    "
            # look like this:
            #   "\\server\so   me\pl-ace\perl\bin\perl.exe"
            my $temp_script = $0;
            $temp_script =~ s/\\/\\\\/g;
            $cmd =~ s/(.*)$temp_script/$1/;
            if($1)
            {
                my $perl = $1;
                $perl =~ s/^\s+//g;           # trim front
                $perl =~ s/\s-.*(?:|\s+)$//g; # trim perl flags ( -d -w ...)
                $perl =~ s/\"//g;             # trim "'s
                $perl =~ s/\s+$//g;           # trim back
                $subst{PERL} = $perl;
                $subst{PERL} .= ".exe" if(!($subst{PERL} =~ /\.exe$/i));
            }
        }

        %subst = (
            %subst,
            AUTOLOGON => 0,
            CFG_FILE  => '',
            KILL      => 'tskill /A /V STAFProc',    #"taskkill /IM STAFProc.exe /F",
            MAP       => $map,
            DIRCACHE_REGPATH  => $dir_cache_regpath,
        );

        if ($map)
        {
            $minwin_user = '/u:'.$minwin_user if($minwin_user);
            if (!$minwin_unc)
            {
                my @maps = list_maps();
                $minwin_drive =~ s/://g;

                foreach my $map (@maps)
                {
                    if (${$map}{'drive'} =~ /$minwin_drive/i)
                    {
                        $minwin_unc = ${$map}{'path'};
                        last;
                    }
                }
            }

            %subst = (%subst,
                      MINWIN_DRIVE  => $minwin_drive,
                      MINWIN_UNC    => $minwin_unc,
                      MINWIN_USER   => $minwin_user,
                      MINWIN_PASSWD => $minwin_passwd,
                      );
        }

        if ($autologon_user && $autologon_domain && $autologon_passwd)
        {
            if ($autologon_disable)
            {
                die "\n-autologon_disable can not be used with "
                  . "-autologon_domain or -autologon_user or "
                  . "-autologon_passwd\n";
            }

            %subst = (%subst, AUTOLOGON => 1,);
        }

        if ($autologon_disable)
        {
            print "\nDisabling autologon. Autologon will be disabled on the next reboot\n";
            my $cmd = "reg add '$autologon_regpath' /v AutoAdminLogon /t REG_SZ /d 0 /f";
            system($cmd);
        }

        %subst = (%subst,
                  AUTOLOGON_DOMAIN  => $autologon_domain,
                  AUTOLOGON_USER    => $autologon_user,
                  AUTOLOGON_PASSWD  => $autologon_passwd,
                  AUTOLOGON_REGPATH => $autologon_regpath,
        );

        my $command;
        my $output;
        my $ver = qx("ver");
        $ver =~ s/version(\D)(\d.\d)/$1 $2/i;
        if($2 < 6) # vista & server2k8 have IPv6 enabled by default
        {
            if(($2 > 5.0) && ($ipv6)) # Win2k and lower don't support IPv6 well
            {
                print "Enabling IPv6\n";
                $command = "netsh interface ipv6 install";
                $output = qx($command);
                chomp($output);
                if($output !~ /^ok\.$/i)
                {
                    if($output =~ qr/^a reboot is required to complete this action\.$/i)
                    {
                        warn $output;
                    }
                    else
                    {
                        die "\"$command\" -- Appears to have Failed with:\n\"$output\"\n".
                            "You need to enable IPv6\n";
                    }
                }
            }
            else
            {
                $subst{CFG_FILE}  = "$staf_dir\\bin\\STAF_IPv4.cfg";
            }
        }

        my $init_script = Windows_init_script();
        $init_script = substitute($init_script, \%subst);
        write_init("$startup\\$service.bat", $init_script);

        $command = "ntrights -u $win_user +r SeAssignPrimaryTokenPrivilege";
        $command = "x:\\WindowsResourceKits\\Tools\\".$command if(-e "x:\\WindowsResourceKits\\Tools\\");
        $output = qx($command);
        if ($? or $output !~ /successful/i)
        {
            warn("Could not add \"replace process level token\" privilege ".
                 "to this user:\r\n" .
                 "  Command: $command\r\n" .
                 "  Output: $output\r\n");
        }
        system("\"$startup\\$service.bat\" upgrade") if($fresh_install);
        my $is_started = !system("tasklist /FI \"IMAGENAME eq STAFProc.exe\" 2>NUL | FIND \"STAFProc.exe\" > NUL 2>\&1");
        return if(!$start);
        if($force || (!$is_started))
        {
            system("\"$startup\\$service.bat\" stop") if($is_started);
            system("\"$startup\\$service.bat\" start");
        }
        else
        {
            die $Already_Started;
        }
    }
    else
    {
        $subst{PERL} = '/usr/software/bin/perl5.14.0' if($subst{PERL} =~ /8/);
        my $bin_path = $staf_dir."/bin";
        my $stafproc = $bin_path."/STAFProc";
        die "Can Not Find " . $stafproc."\n" . `mount` if ((! -e $stafproc) && (!$fresh_install));
        die "Please run this script as root (e.g. using \"sudo\")\n" if($> != 0);

        # right now passing anything other than "root" to unix_user only works on linux
        #if (($^O !~ /linux/i) && ($unix_user ne "root"))
        #{
        #    die "Setting the unixuser for OS '$^O' is not currently supported\n";
        #}

        # make sure the user we are trying to use for $unix_user exists
        my $unix_uid = getpwnam($unix_user);
        my $unix_gid = (getpwnam($unix_user))[3];

        if (! defined $unix_uid)
        {
            die "Please verify that unix user '$unix_user' exists prior to running this script.\n";
        }

        $ENV{PATH} = $ENV{PATH}.":$bin_path";
        my $bash;

        if($shell)
        {
            $bash = $shell;
        }
        else
        {
            chomp($bash = `which bash`);
            $bash='/bin/bash' if (! -e $bash);
            $bash='/usr/software/bin/bash' if (! -x $bash);
        }
        my $staf_tmp_dir = "/var/staf";
        # Unix or Unix-like
        %subst =  ( %subst,
                    SH              => $bash,
                    ENVSIZE         => "",
                    STAF_TEMP_DIR   => $staf_tmp_dir,
                    EQUAL           => "==",
                    CFG_FILE        => "$bin_path/STAF_IPv4.cfg",
                    PATH            => join(":",qw(
                                                    /usr/software/bin
                                                    /usr/software/test/bin
                                                    /usr/software/test/noarch/bin
                                                    /usr/software/utils/bin
                                                    /usr/software/rats/bin
                                                    /usr/software/sbin
                                                    /usr/bin
                                                    /usr/sbin
                                                    /sbin
                                                    /bin
                                                    /usr/local/bin
                                                    /usr/local/sbin
                                                    )),
                    );

        $subst{LD_LIBRARY_PATH}.="$staf_dir/lib:$staf_dir/lib/perl514";

        my $initd;
        my $init_script;

        use IPC::Open3;
        open3(\*CHLD_IN, \*CHLD_OUT, \*CHLD_OUT, "su - $unix_user");
        print CHLD_IN "echo Oogally_Boogally\n";
        print CHLD_IN "printenv PATH\n" if ($^O =~ /linux/i);
        print CHLD_IN "echo \$PATH\n"   if ($^O =~ /solaris|aix/i);
        print CHLD_IN "exit\n";
        local $|=1;
        my $temp =<CHLD_OUT>;
        close(CHLD_IN);
        close(CHLD_OUT);
        chomp($temp);
        $temp =~ s/Oogally_Boogally\s(.*)/$1/;
        my $root_path = $1;

        my @a = split(":",$root_path) if($root_path);
        my @b = split(":",$subst{PATH});
        my @c;
        foreach my $var (@a)
        {
            push(@c, $var) if!(grep{$var eq $_}@b);
        }
        $root_path = join(":",@c);
        $root_path =~ s/ /\\ /g;
        $subst{PATH} = $bin_path . ":" . $subst{PATH} . ":" . $root_path;

        my $osver = $Config{osvers};
        
        my @files = ('DataDir_-var-staf-data.tmp', 'STAFIPC_STAF', 'STAF.tmp');
        my @dirs = ("/tmp", $staf_tmp_dir);
        foreach my $file (@files)
        {
            my $location1 = $dirs[0] . "/$file";
            my $location2 = $dirs[1] . "/$file";
            if((-e $location1) && (! -e $location2))
            {
                symlink($location1, $location2);
            }
            if(( -e $location2) && (! -e $location1))
            {
                symlink($location2, $location1);
            }
        }

        if ($^O =~ /linux/i)
        {
            if($ipv6)
            {
                chomp(my $is_ipv6_enabled = `/sbin/lsmod |grep -w ipv6`);
                if ($is_ipv6_enabled !~ qr/ipv6/)
                {
                    # Is the Kernel capable?
                    # 2.6 is "recommended", according to:
                    #   http://www.ibiblio.org/pub/Linux/docs/HOWTO/other-formats/html_single/Linux+IPv6-HOWTO.html#CHAPTER-SYSTEMCHECK
                    $osver =~ s/(-.*)//;
                    my @ver = split(/\./, $osver);
                    $osver = $ver[0] . "." . $ver[1];
                    $ipv6 = 0 if($osver < 2.6);
                    # Is the Module available?
                    $ipv6 = 0 if(($ipv6) && (system("/sbin/modprobe ipv6") != 0));
                }
            }

            $subst{CFG_FILE}  = "" if($ipv6);
            $subst{PROC_TEST} = "/bin/ps -C STAFProc > /dev/null";

            my $has_chkconfig = 1;
            $subst{SERVICE_HEADER} = "# chkconfig: 345 99 99";
            if (-e "/etc/redhat-release")
            {
                $initd = "/etc/rc.d/init.d/";
                $subst{KILL} = "killproc STAFProc";
                $subst{FUNCTIONS} = "/etc/rc.d/init.d/functions";
                $subst{SUCCESS} = "success";
            }
            elsif (-e "/etc/SuSE-release" or (-e "/etc/os-release" and `cat /etc/os-release` =~ /SUSE Linux/i)) {
                $initd = "/etc/init.d/";
                $subst{KILL} = "/usr/bin/pkill -9 STAFProc";
                $subst{FUNCTIONS} = "/etc/rc.status";
                $subst{SUCCESS} = "echo success";
            }
            elsif (-e "/etc/lsb-release")
            {
                warn("THIS IS AN UNSUPPORTED DISTRIBUTION\n... Good Luck ...\n\n");
                $subst{SERVICE_HEADER} = join("\n",("### BEGIN INIT INFO",
                                                     "# Provides:             STAFProc",
                                                     "# Required-Start:       ",
                                                     "# Required-Stop:        ",
                                                     "# Default-Start:        3 4 5",
                                                     "# Default-Stop:         99",
                                                     "# Short-Description:    SOFTWARE TEST AUTOMATION FRAMWORK",
                                                     "### END INIT INFO"));
                $has_chkconfig = 0;
                $subst{CHKCFG} = "";
                #invoke-rc.d
                #update-rc.d
                $initd = "/etc/init.d/";
                $subst{KILL} = "/usr/bin/pkill -9 STAFProc";
                $subst{FUNCTIONS} = "/lib/lsb/init-functions";
                $subst{SUCCESS} = "echo -n success\n";
            }
            else
            {
                die "Unknown distribution of Linux.\nThis script only knows how to set up init scripts for RedHat.\n";
            }
            $subst{SPECIAL_USER} = "su -m -s /bin/sh $unix_user -c ";

            $init_script = Redhat_init_script();
            $init_script = substitute($init_script, \%subst);
            #print $init_script;
            write_init($initd . $service, $init_script);
            chkconfig($service) if($has_chkconfig);
            update_rcd($service) if(!$has_chkconfig);
            my $is_staf_started = !system($subst{PROC_TEST});

            die $Already_Started if(($is_staf_started) && (!$force));

            if ( -e '/etc/fedora-release' ) {
                if(($is_staf_started) && ($start)) {
                    print "Stopping STAFProc:";
                    system("systemctl stop netapp_staf.service");
                    if($? == 0) { print "\t[OK]\n" }
                    else { print "\t[FAILED]"; exit 1; }
                }
                change_local_logs($unix_uid, $unix_gid, $staf_datadir_symlink);
                print "Starting STAFProc:";
                system("systemctl start netapp_staf.service") if($start);
                if($? == 0) { print "\t[OK]\n" }
                else { print "\t[FAILED]"; exit 1; }
            }
            else {
                system($initd . $service . " stop") if(($is_staf_started) && ($start));
                change_local_logs($unix_uid, $unix_gid, $staf_datadir_symlink);
                if($start)
                {
                    system($initd . $service . " start");
                    system("$staf_dir/bin/staf local var set system var STAF/Env/USERNAME=$unix_user") unless $?;
                }
            }
        }
        elsif ($^O =~ /solaris/i)
        {
            # 2.8 == 5.8, 2.10 == 5.10?  WTF?
            chomp($osver = `uname -r`);
            my @version_parts = split('\.', $osver);
            $subst{CFG_FILE} = "" if($ipv6);
            #$subst{PROC_TEST} = "/usr/bin/ps -u $unix_user | grep STAFProc";
            $subst{PROC_TEST} = "/usr/bin/ps -a | grep STAFProc";
            my $standard_staf_datadir = "/var/staf";
            mkpath($standard_staf_datadir) if(! -d $standard_staf_datadir);
            if($version_parts[1] >= 10)
            {
                # try to get rid of the old service first
                # XXX: This should be removed some time in the future
                if(-e "/etc/init.d/$service")
                {
                    eval{system("/etc/init.d/$service stop");};
                    foreach my $dir ('/etc/rc3.d', '/etc/rc4.d', '/etc/rc5.d')
                    {
                        eval{unlink("$dir/S99$service");};
                    }
                    eval{unlink("/etc/init.d/$service");};
                }

                # now, do the new svc thingy
                $initd = "/lib/svc/method/";
                $subst{KILL} = "/usr/bin/pkill -9 STAFProc";
                $subst{FUNCTIONS} = "/lib/svc/share/smf_include.sh";
                $subst{SUCCESS} = "echo success";
                my $manifest = ORM_script();
                my $manifest_file = "/var/svc/manifest/application/$service.xml";
                $init_script = substitute(smf_init_script(), \%subst);
                #print $init_script;
                write_init($manifest_file, $manifest);
                write_init($initd . "STAF", $init_script);

                my $is_started = !system("/usr/bin/svcs | grep $service| grep online");
                system("logadm -e dl-nate-dev\@netapp.com -N -w netapp_staf -C 8 -s 5m -S 100m /var/svc/log/application-netapp_staf:default.log");
                die $Already_Started if(($is_started) && (!$force));
                if($start)
                {
                    system("/usr/sbin/svccfg validate ".$manifest_file);
                    system("/usr/sbin/svccfg import ".$manifest_file);
                    system("/usr/sbin/svcadm disable -s svc:/application/netapp_staf");
                    system("/usr/sbin/svcadm -v enable svc:/application/netapp_staf");
                    sleep(5);
                    system("/usr/sbin/svcadm clear svc:/application/netapp_staf 2> /dev/null");
                    for (0..180)
                    {
                        exit(0) if(system(catdir($initd,"STAF"). " status") == 0);
                        sleep 1;
                    }
                    exit 1;
                }
            }
            else
            {
                $subst{KILL} = "/usr/bin/pkill -x -u 0 STAFProc";
                #$subst{KILL} = "/usr/bin/pkill -x STAFProc";
                _setup_generic_unix("/etc", $service, \%subst,$unix_uid, $unix_gid, $staf_datadir_symlink);
            }
        }
        elsif ($^O =~ /^aix$/i)
        {
            my @version_parts = split('\.', $osver);
            $osver = $version_parts[0] . "." . $version_parts[1];
            die "$osver is not yet supported on $^O" if($osver < 5.1);
            my $standard_staf_datadir = "/var/staf";
            mkpath($standard_staf_datadir) if(! -d $standard_staf_datadir);
            $subst{PROC_TEST} = "/usr/bin/ps -u $unix_user | grep STAFProc";
            #making the Max ENV or ARG size = 4096*48 bytes (max ncargs value is 128)
            $subst{ENVSIZE} = "chdev -l sys0 -a ncargs=64";
            $subst{KILL} = 'pid=`/usr/bin/ps -e |/usr/bin/grep "STAFProc" |/usr/bin/sed -e \'s/^  *//\' -e \'s/ .*//\'` &&[ ! -z "$pid" ] && /usr/bin/kill -9 $pid &';
            _setup_generic_unix("/etc/rc.d", $service, \%subst,$unix_uid, $unix_gid, $staf_datadir_symlink);
        }
        elsif ($^O =~ /^hpux$/i)
        {
            $subst{EQUAL} = "=";
            chmod(0777,mkpath("/var/log"));
            $subst{SH} = "/usr/bin/sh";
            my @version_parts = split('\.', $osver);
            $osver = $version_parts[0] . "." . $version_parts[1];
            #die "$osver is not yet supported on $^O" if($osver < 5.1);
            my $standard_staf_datadir = "/var/staf";
            mkpath($standard_staf_datadir) if(! -d $standard_staf_datadir);
            $subst{PROC_TEST} = "/usr/bin/ps -u $unix_user | grep STAFProc";
            $subst{KILL} = 'pid=`/usr/bin/ps -e |/usr/bin/grep "STAFProc" |/usr/bin/sed -e \'s/^  *//\' -e \'s/ .*//\'` &&[ ! -z "$pid" ] && /usr/bin/kill $pid &';
            _setup_generic_unix("/sbin", $service, \%subst,$unix_uid, $unix_gid, $staf_datadir_symlink);
        }
        else
        {
            die "Automatic installation is not yet supported for operating system \"$^O\"\n";
        }
    }
}

#Solaris 5.8 and aix
sub _setup_generic_unix
{
    my ($initd_path, $service, $subst, $unix_uid, $unix_gid, $staf_datadir_symlink) = @_;
    my $init_script = Unix_init_script();
    $init_script = substitute($init_script, $subst);
    write_init("$initd_path/init.d/$service", $init_script);
    foreach my $dir ($initd_path.'/rc3.d', $initd_path.'/rc4.d', $initd_path.'/rc5.d')
    {
        # stinky 'ol Slowaris doesn't always have rc4&5, so don't die
        eval
        {
            unlink("$dir/S99$service");
            symlink("../init.d/$service","$dir/S99$service");
        };
        if($@)
        {
            print "Could not create symlink $dir/S99$service: $!\n";
        }
    }

    my $is_staf_started = !system($subst->{PROC_TEST});
    die $Already_Started if(($is_staf_started) && (!$force));
    if($start)
    {
        system("$initd_path/init.d/$service stop") if($is_staf_started);
        sleep(10); # XXX: we should verify it's stopped
        #change_local_logs($unix_uid, $unix_gid, $staf_datadir_symlink);
        system("$initd_path/init.d/$service start");
        system("$staf_dir/bin/staf local var set system var STAF/Env/USERNAME=$unix_user") unless $?;
    }
}

sub change_local_logs
{
    my ($unix_uid, $unix_gid, $staf_datadir_symlink) = @_;

    # we need to create a datadir that is different than the normal STAF datadir and then
    # symlink /var/staf to it.   we also need to make sure it is owned by the appropriate user
    if($start)
    {
        system("rm -f /tmp/DataDir_-var-staf-data.tmp");
        system("rm -f /var/staf/DataDir_-var-staf-data.tmp");
        system("rm -f /tmp/STAFIPC_STAF");
        system("rm -f /var/staf/STAFIPC_STAF");
        system("rm -f /tmp/STAF.tmp");
        system("rm -f /var/staf/STAF.tmp");
    }

    my $standard_staf_datadir = "/var/staf";
    my $staf_datadir = $standard_staf_datadir;
    $staf_datadir = $staf_datadir_symlink if(defined($staf_datadir_symlink));

    mkpath($staf_datadir) if(! -d $staf_datadir);

    system("chown -R $unix_uid:$unix_gid $staf_datadir") &&
        die "Failed to change the owner of '$staf_datadir' to '$unix_user' '$unix_uid' '$^E'\n";

    if ( defined $staf_datadir_symlink )
    {
        # the normal staf datadir already exists
        if ( -e $standard_staf_datadir )
        {
            print "STAF DATADIR '$standard_staf_datadir' already exists.  " .
                "Preserving in '$standard_staf_datadir.orig'\n";
            rename($standard_staf_datadir, "$standard_staf_datadir.orig") ||
                die "Unable to save original STAF DATADIR '$standard_staf_datadir' '$^E'\n";
        }
        # create a symlink to the datadir so that STAF can start up properly
        symlink($staf_datadir, $standard_staf_datadir) ||
            die "Unable to create symlink from '$staf_datadir' to '$standard_staf_datadir' '$^E'\n";

        if ( -e '$standard_staf_datadir.orig')
        {
            system("cp -r -L $standard_staf_datadir.orig/* $standard_staf_datadir");
        }
    }

    # handle getting the log file setup properly
    if(-e "/var/log/STAFProc.log")
    {
        chown($unix_uid, $unix_gid, "/var/log/STAFProc.log") ||
            die "Unable to change ownership of STAFProc log file '/var/log/STAFProc.log'" .
                " to '$unix_uid:$unix_gid' '$^E'\n";
    }
}

# chkconfig
sub chkconfig
{
    my $service = shift;
    system("/sbin/chkconfig --add $service");
    system("/sbin/chkconfig --level 345 $service on");
}

# update_rcd
sub update_rcd
{
    my $service = shift;
    system("update-rc.d $service defaults 99 > /dev/null");
    system("update-rc.d $service start 99 3 4 5 . stop 99 3 4 5 . > /dev/null");
}

sub ORM_script
{
return <<'EOT';
<?xml version="1.0"?>
<!DOCTYPE service_bundle
  SYSTEM "/usr/share/lib/xml/dtd/service_bundle.dtd.1">

<service_bundle type='manifest' name='netapp_staf'>
    <service
        name='application/netapp_staf'
        type='service'
        version='1'>
        <create_default_instance enabled='false' />
        <single_instance />

        <dependency
            name='multi-user-server'
            grouping='optional_all'
            type='service'
            restart_on='none'>
            <service_fmri value='svc:/milestone/multi-user-server' />
        </dependency>

        <exec_method
            type='method'
            name='start'
            exec='/lib/svc/method/STAF start'
            timeout_seconds='60'>
        <method_context>
            <method_credential user='root' />
        </method_context>
        </exec_method>

        <exec_method
            type='method'
            name='test'
            exec='/lib/svc/method/STAF status'
            timeout_seconds='60'>
        <method_context>
            <method_credential user='root' />
        </method_context>
        </exec_method>

        <exec_method
            type='method'
            name='refresh'
            exec='/lib/svc/method/STAF status'
            timeout_seconds='60'>
        <method_context>
            <method_credential user='root' />
        </method_context>
        </exec_method>

        <exec_method
            type='method'
            name='restart'
            exec='/lib/svc/method/STAF restart'
            timeout_seconds='60'>
        <method_context>
            <method_credential user='root' />
        </method_context>
        </exec_method>

        <exec_method
            type='method'
            name='stop'
            exec='/lib/svc/method/STAF stop'
            timeout_seconds='60' >
        <method_context>
            <method_credential user='root' />
        </method_context>
        </exec_method>

        <property_group name='startd' type='framework'>
            <propval name='ignore_error' type='astring' value='core,signal' />
            <propval name='duration' type='astring' value='contract' />
        </property_group>

        <template>
            <common_name>
                <loctext xml:lang='C'>
                    STAF Service
                </loctext>
            </common_name>
        </template>
    </service>
</service_bundle>
EOT
}

# write_init
sub write_init
{
    my ($filename, $netapp_staf) = @_;
    unlink($filename) if(-e $filename);
    my $fh;
    open($fh, '>', $filename) or die "Could not open $filename: $!";
    print $fh $netapp_staf or die "Could not write to $filename: $!";
    close($fh) or die "Could not close $filename: $!";

    chmod(0755, $filename) or die "Could not chmod $filename: $!";
}

# Do variable substitution in $text.  For every key KEY in %$subst,
# replace "@KEY@" in the $text with the value in %$subst for that key.

sub substitute
{
    my ($text, $subst) = @_;
    while (my ($key, $value) = each %$subst)
    {
        $text =~ s/\@$key\@/$value/g;
    }
    return $text;
}

sub list_maps() {

    my @maps;
    return eval {
        my @output = `net use`;
        foreach my $line (@output) {
            chomp($line);
            my ($rest, $path, $server, $share) =
              ($line =~ /(.*)\s+(\\\\([^\\]+)\\(\S+))/);
            next unless ($path);
            my ($status, $drive) = split(/\s+/, $rest);
            my $map = {server => $server,
                       share  => $share,
                       drive  => $drive,
                       path   => $path
                       };
            $map->{status} = $status if ($status);
            push(@maps, $map);
        }
        return (@maps);
    };
}

sub Windows_init_script
{
    return <<'EOT';
@echo off

:: the owner of this script is dl-nate-dev@netapp.com

CALL :BEGIN %*
GOTO :EOF

:IS_RUNNING
    CALL @STAFDIR@\bin\STAF.exe local ping ping 2>NUL | FIND "PONG" > %SYSTEMDRIVE%\is_staf_started
    :: tasklist /FI "IMAGENAME eq STAFProc.exe" 2>NUL | FIND "STAFProc.exe" > %SYSTEMDRIVE%\is_staf_started
    FC %SYSTEMDRIVE%\_empty %SYSTEMDRIVE%\is_staf_started /L > NUL
    GOTO :EOF

:START
    CMD /C %SYSTEMDRIVE%
    CALL :IS_RUNNING > NUL
    IF %ERRORLEVEL% == 0 (GOTO :NOT_STARTED) ELSE (
        IF EXIST "%SYSTEMDRIVE%\is_staf_started" (DEL  /F /Q %SYSTEMDRIVE%\is_staf_started)
        IF NOT EXIST "%SYSTEMDRIVE%\staf_is_running" (CALL @echo > %SYSTEMDRIVE%\staf_is_running)
        echo STAFProc is Already Running
        GOTO :EOF
    )
    :NOT_STARTED
        IF @UPDATE@ == 1 (CALL :UPDATE)
        CALL :LOOP_STAFDIR_EXISTS
        IF %_LOOP_STAFDIR_EXISTS% == 0 (
            @echo ERROR: @STAFDIR@ does not exist.
            GOTO :EOF
        )
        IF EXIST %SYSTEMDRIVE%\STAFProc_5.log (DEL  /F /Q %SYSTEMDRIVE%\STAFProc_5.log)
        IF EXIST %SYSTEMDRIVE%\STAFProc_4.log (MOVE /Y    %SYSTEMDRIVE%\STAFProc_4.log %SYSTEMDRIVE%\STAFProc_5.log > NUL 2>&1)
        IF EXIST %SYSTEMDRIVE%\STAFProc_3.log (MOVE /Y    %SYSTEMDRIVE%\STAFProc_3.log %SYSTEMDRIVE%\STAFProc_4.log > NUL 2>&1)
        IF EXIST %SYSTEMDRIVE%\STAFProc_2.log (MOVE /Y    %SYSTEMDRIVE%\STAFProc_2.log %SYSTEMDRIVE%\STAFProc_3.log > NUL 2>&1)
        IF EXIST %SYSTEMDRIVE%\STAFProc_1.log (MOVE /Y    %SYSTEMDRIVE%\STAFProc_1.log %SYSTEMDRIVE%\STAFProc_2.log > NUL 2>&1)
        IF EXIST %SYSTEMDRIVE%\STAFProc.log   (MOVE /Y    %SYSTEMDRIVE%\STAFProc.log   %SYSTEMDRIVE%\STAFProc_1.log > NUL 2>&1)

        ::@echo *
        ::@echo STAFProc is now going to start.  Leave this window up.
        ::@echo *
        ::@STAFDIR@\bin\STAFProc.exe @CFG_FILE@ > %SYSTEMDRIVE%\STAFProc.log 2>&1
        echo x > %SYSTEMDRIVE%\STAFProc.log
        IF NOT %ERRORLEVEL% == 0 (
            CALL :INSTALL
            GOTO :EOF
        )

        START /B @STAFDIR@\bin\STAFProc.exe @CFG_FILE@ > %SYSTEMDRIVE%\STAFProc.log 2>&1

        CALL :LOOP_IS_RUNNING 0
        IF %_LOOP_IS_RUNNING% == 0 (
            @echo *
            @echo STAFProc is now Started.  Leave this window up.
            @echo *
            IF NOT EXIST "%SYSTEMDRIVE%\staf_is_running" (CALL @echo > %SYSTEMDRIVE%\staf_is_running)
            @STAFDIR@\bin\STAF.exe local queue get wait
        ) ELSE ( echo STAFProc COULD NOT BE STARTED )

    GOTO :EOF

:STOP
    CALL :IS_RUNNING > NUL
    IF %ERRORLEVEL% == 1 (
        TSKILL /A /V STAFProc > NUL 2>&1
        CALL :LOOP_IS_RUNNING 21
    )
    IF EXIST "@STAFDIR@\staf_is_running" (DEL /F /Q @STAFDIR@\staf_is_running)
    GOTO :EOF

:LOOP_IS_RUNNING
    SET _LOOP_IS_RUNNING=1
    SETLOCAL
    SET i=0
    SET REPEATS=240
    :LOOP
        CALL :SLEEP 1
        CALL @STAFDIR@\bin\STAF.exe local ping ping > NUL
        IF %ERRORLEVEL% == %1% (
            CALL :SLEEP 2
            ENDLOCAL
            SET _LOOP_IS_RUNNING=0
            GOTO :EOF
        )
        SET /A i=%i%+1
        IF %i% LSS %REPEATS% (GOTO :LOOP)
    ENDLOCAL
    SET _LOOP_IS_RUNNING=1
    GOTO :EOF

:LOOP_STAFDIR_EXISTS
    SET _LOOP_STAFDIR_EXISTS=1
    SETLOCAL
    SET i=0
    SET REPEATS=240
    IF EXIST @STAFDIR@ (GOTO :EOF)
    :LOOP
        CALL :SLEEP 1
        IF NOT EXIST @STAFDIR@ (
            CALL :SLEEP 2
            ENDLOCAL
            SET _LOOP_STAFDIR_EXISTS=0
            GOTO :EOF
        )
        SET /A i=%i%+1
        IF %i% LSS %REPEATS% (GOTO :LOOP)
    ENDLOCAL
    SET _LOOP_IS_RUNNING=1
    GOTO :EOF

:UPDATE
    IF NOT EXIST "@INSTALL_FROM@" (
        CALL @echo > @STAFDIR@\staf_is_installed
        GOTO :EOF
    )
    FC /L @INSTALL_FROM@\* @STAFDIR@\* > NUL 2>&1
    IF NOT %ERRORLEVEL% == 0 (
        CALL :INSTALL
        GOTO :EOF
    )
    FC /L @INSTALL_FROM@\bin\* @STAFDIR@\bin\* > NUL 2>&1
    IF NOT %ERRORLEVEL% == 0 (
        CALL :INSTALL
        GOTO :EOF
    )
    FC /L @INSTALL_FROM@\lib\* @STAFDIR@\lib\* > NUL 2>&1
    IF NOT %ERRORLEVEL% == 0 (
        CALL :INSTALL
        GOTO :EOF
    )
    FC /L @INSTALL_FROM@\codepage\* @STAFDIR@\codepage\* > NUL 2>&1
    IF NOT %ERRORLEVEL% == 0 (
        CALL :INSTALL
        GOTO :EOF
    )
    FC /L @INSTALL_FROM@\include\* @STAFDIR@\include\* > NUL 2>&1
    IF NOT %ERRORLEVEL% == 0 (
        CALL :INSTALL
        GOTO :EOF
    )
    CALL @echo > @STAFDIR@\staf_is_installed
    GOTO :EOF

:INSTALL
    IF "@INSTALL_FROM@" NEQ "@STAFDIR@" (
        CALL RMDIR @STAFDIR@ /S /Q
        IF %ERRORLEVEL% NEQ 0 (
            echo RMDIR error = %ERRORLEVEL%
            CALL :INSTALL
            GOTO :EOF
        )
        CALL XCOPY @INSTALL_FROM@ @STAFDIR@ /R /I /H /K /E /Y
        IF %ERRORLEVEL% NEQ 0 (
            echo XCOPY error = %ERRORLEVEL%
            CALL :INSTALL
            GOTO :EOF
        )
    )
    CALL @echo > @STAFDIR@\staf_is_installed
    GOTO :EOF

:SLEEP
    FOR /l %%a in (%~1,-1,1) DO (ping -n 2 -w 1 localhost > NUL 2>&1)
    GOTO :EOF

:REWRITE
    IF EXIST "@PERL@" (IF EXIST "@START_STAF@" (START /B @PERL@ @START_STAF@ @STARTUP_ARGS@ > NUL 2>&1))
    GOTO :EOF

:BEGIN

    echo.

    IF "@MAP@" EQU "1" (
        IF NOT EXIST @MINWIN_DRIVE@:\ echo Mapping @MINWIN_DRIVE@ to @MINWIN_UNC@
        IF NOT EXIST @MINWIN_DRIVE@:\ net use @MINWIN_DRIVE@: @MINWIN_UNC@ @MINWIN_USER@ @MINWIN_PASSWD@
    )

    IF "@AUTOLOGON@" EQU "1" (
        echo Configuring autologon
        reg add "@AUTOLOGON_REGPATH@" /v AutoAdminLogon /t REG_SZ /d 1 /f
        reg add "@AUTOLOGON_REGPATH@" /v DefaultUserName /t REG_SZ /d @AUTOLOGON_USER@ /f
        reg add "@AUTOLOGON_REGPATH@" /v DefaultPassword /t REG_SZ /d @AUTOLOGON_PASSWD@ /f
        reg add "@AUTOLOGON_REGPATH@" /v DefaultDomainName /t REG_SZ /d @AUTOLOGON_DOMAIN@ /f
    )
    reg add "@DIRCACHE_REGPATH@" /v DirectoryCacheLifetime /t REG_DWORD /d 0 /f
    reg add "@DIRCACHE_REGPATH@" /v FileNotFoundCacheLifetime /t REG_DWORD /d 0 /f
    reg add "@DIRCACHE_REGPATH@" /v FileInfoCacheLifetime /t REG_DWORD /d 0 /f

    IF NOT EXIST "%SYSTEMDRIVE%\_empty"      (CALL @echo > %SYSTEMDRIVE%\_empty)
    IF EXIST "%SYSTEMDRIVE%\staf_is_started" (DEL /F /Q %SYSTEMDRIVE%\staf_is_started)
    IF EXIST "%SYSTEMDRIVE%\staf_is_running" (DEL /F /Q %SYSTEMDRIVE%\staf_is_running)
    IF EXIST "%SYSTEMDRIVE%\is_staf_started" (DEL /F /Q %SYSTEMDRIVE%\is_staf_started)
    IF EXIST "@STAFDIR@\staf_is_installed"   (DEL /F /Q @STAFDIR@\staf_is_installed)
    CMD /C %SYSTEMDRIVE%
    SET PATH=@STAFDIR@\bin;%PATH%
    SET STAFCONVDIR=@STAFDIR@\codepage
    SET STAF_DIR=@STAFDIR@
    SET STAF_INSTANCE_NAME=STAF
    CALL :IS_RUNNING > NUL
    IF "%1" EQU "start" (
        CALL :START
        GOTO :EOF
    )
    IF "%1" EQU "stop" (
        CALL :STOP
        GOTO :EOF
    )
    IF "%1" EQU "upgrade" (
        IF %ERRORLEVEL% == 0 (
            CALL :UPDATE
            GOTO :EOF

        ) ELSE (
            CALL :STOP
            CALL :UPDATE
            CALL :START
            GOTO :EOF
        )
        GOTO :EOF
    )
    IF "%1" EQU "version" (
        echo @VERSION@
        GOTO :EOF
    )
    ::IF "%1" EQU "status" (
    ::    CALL :IS_RUNNING > NUL
    ::    IF %ERRORLEVEL% == 0 (
    ::        IF EXIST "%SYSTEMDRIVE%\is_staf_started" (DEL  /F /Q %SYSTEMDRIVE%\is_staf_started)
    ::        IF EXIST "%SYSTEMDRIVE%\staf_is_running" (DEL  /F /Q %SYSTEMDRIVE%\staf_is_running)
    ::        echo STAF is Not running
    ::        GOTO :EOF
    ::    )
    ::    ELSE (
    ::        IF EXIST "%SYSTEMDRIVE%\is_staf_started" (DEL  /F /Q %SYSTEMDRIVE%\is_staf_started)
    ::        IF NOT EXIST "%SYSTEMDRIVE%\staf_is_running" (CALL @echo > %SYSTEMDRIVE%\staf_is_running)
    ::        echo STAF is running
    ::        GOTO :EOF
    ::    )
    ::)
    CALL :REWRITE
    CALL :START
    GOTO :EOF
:END
EOT
}

# Unix_init_script
# for AIX, solaris8, maybe HPUX
sub Unix_init_script
{
    return <<'EOT';
#! @SH@

# owner: dl-nate-dev@netapp.com

start()
{
    if [ -e @PERL@ ] && [ -e @START_STAF@ ]; then
        @PERL@ @START_STAF@ @STARTUP_ARGS@ > /dev/null  2>&1
    fi
    if [ @UPDATE@ @EQUAL@ 1 ]; then
        update
    fi
    stafproc_exists
    if [ $? != 1 ]; then
        return 1
    fi
    @ENVSIZE@
    PATH=@PATH@
    export PATH
    LD_LIBRARY_PATH=@LD_LIBRARY_PATH@
    export STAF_TEMP_DIR=@STAF_TEMP_DIR@
    export LD_LIBRARY_PATH
    export STAF_DIR=@STAFDIR@
    export STAFCONVDIR=@STAFDIR@/codepage
    export STAF_DEBUG_RC_21=1
    status > /dev/null
    if [ $? @EQUAL@ 0 ]; then
        echo STAFProc is Already Running
        return 0
    fi
    rm -f /tmp/DataDir_-var-staf-data.tmp
    rm -f @STAF_TEMP_DIR@/DataDir_-var-staf-data.tmp
    rm -f /tmp/STAFIPC_STAF
    rm -f @STAF_TEMP_DIR@/STAFIPC_STAF
    rm -f /tmp/STAF.tmp
    rm -f @STAF_TEMP_DIR@/STAF.tmp

    rm -f /var/log/STAFProc_5.log
    mv -f /var/log/STAFProc_4.log /var/log/STAFProc_5.log > /dev/null 2>&1
    mv -f /var/log/STAFProc_3.log /var/log/STAFProc_4.log > /dev/null 2>&1
    mv -f /var/log/STAFProc_2.log /var/log/STAFProc_3.log > /dev/null 2>&1
    mv -f /var/log/STAFProc_1.log /var/log/STAFProc_2.log > /dev/null 2>&1
    mv -f /var/log/STAFProc.log   /var/log/STAFProc_1.log > /dev/null 2>&1

    cd /
    ulimit -c unlimited
    ulimit -v unlimited
    ulimit -n unlimited
    ulimit -s unlimited
    echo "starting STAFProc"
    nohup @STAFDIR@/bin/STAFProc @CFG_FILE@ > /var/log/STAFProc.log 2>&1 &
    i=0
    REPEATS=30
    while [ $i -lt $REPEATS ]
        do
            sleep 1
            status > /dev/null
            if [ $? @EQUAL@ 0 ]; then
                sleep 2
                i=$REPEATS
            fi
        i=`expr $i + 1`
    done

    status > /dev/null
    if [ $? != 0 ]; then
        start
    fi

    if [ -e /tmp/DataDir_-var-staf-data.tmp ]; then
        if [ ! -e @STAF_TEMP_DIR@/DataDir_-var-staf-data.tmp ]; then
            ln -s /tmp/DataDir_-var-staf-data.tmp @STAF_TEMP_DIR@/DataDir_-var-staf-data.tmp
        fi
    fi
    if [ -e /tmp/STAFIPC_STAF ]; then
        if [ ! -e @STAF_TEMP_DIR@/STAFIPC_STAF ]; then
            ln -s /tmp/STAFIPC_STAF @STAF_TEMP_DIR@/STAFIPC_STAF
        fi
    fi
    if [ -e /tmp/STAF.tmp ]; then
        if [ ! -e @STAF_TEMP_DIR@/STAF.tmp ]; then
            ln -s /tmp/STAF.tmp @STAF_TEMP_DIR@/STAF.tmp
        fi
    fi
    if [ -e @STAF_TEMP_DIR@/DataDir_-var-staf-data.tmp ]; then
        if [ ! -e /tmp/DataDir_-var-staf-data.tmp ]; then
            ln -s @STAF_TEMP_DIR@/DataDir_-var-staf-data.tmp /tmp/DataDir_-var-staf-data.tmp
        fi
    fi
    if [ -e @STAF_TEMP_DIR@/STAFIPC_STAF ]; then
        if [ ! -e /tmp/STAFIPC_STAF ]; then
            ln -s @STAF_TEMP_DIR@/STAFIPC_STAF /tmp/STAFIPC_STAF
        fi
    fi
    if [ -e @STAF_TEMP_DIR@/STAF.tmp ]; then
        if [ ! -e /tmp/STAF.tmp ]; then
            ln -s @STAF_TEMP_DIR@/STAF.tmp /tmp/STAF.tmp
        fi
    fi

    is_stafproc_ready
    if [ $? == 0 ]; then
        return 1;
    fi
}

is_stafproc_ready()
{
    i=0
    # 4 minutes
    REPEATS=240
    @STAFDIR@/bin/STAF local REAPER VERSION > /dev/null 2>&1
    if [ $? != 0 ]; then
        echo
        echo "Please wait.  This may take up to 4 minutes"
    fi
    while [ $i -lt $REPEATS ]
        do
            @STAFDIR@/bin/STAF local REAPER VERSION > /dev/null 2>&1
            if [ $? == 0 ]; then
                # true
                return 1
            fi
            sleep 1
            i=`expr $i + 1`
    done
    echo "It appears that a Service has not started"
    echo `cat /var/log/STAFProc.log`
    # false
    return 0
}

update()
{
    if [ -e @INSTALL_FROM@ ]; then
        diff @INSTALL_FROM@/install.properties @STAFDIR@/install.properties > /dev/null 2>&1
        if [ $? != 0 ]; then
            install
        fi
    fi
}

install()
{
    if [ "@INSTALL_FROM@" != "@STAFDIR@" ]; then
        rm -rf @STAFDIR@
        cp -r @INSTALL_FROM@ @STAFDIR@
    fi
}

# Had to call this '_stop' because HP-UX was bitching about 'stop'
_stop()
{
    @KILL@
    i=0
    REPEATS=30
    while [ $i -lt $REPEATS ]
        do
            sleep 1
            status > /dev/null
            if [ $? != 0 ]; then
                sleep 2
                i=$REPEATS
            fi
            i=`expr $i + 1`
    done
    rm -f /tmp/DataDir_-var-staf-data.tmp
    rm -f @STAF_TEMP_DIR@/DataDir_-var-staf-data.tmp
    rm -f /tmp/STAFIPC_STAF
    rm -f @STAF_TEMP_DIR@/STAFIPC_STAF
    rm -f /tmp/STAF.tmp
    rm -f @STAF_TEMP_DIR@/STAF.tmp
}

status()
{
    @PROC_TEST@
    return $?
}

stafproc_exists()
{
    i=0
    REPEATS=60
    while [ $i -lt $REPEATS ]
        do
            sleep 1
            if [ -e @STAFDIR@/bin/STAFProc ]; then
                return 1
            fi
            i=`expr $i + 1`
    done
    return 0
}

case "$1" in
    'start')
        start
        ;;

    'stop')
        echo "killing  STAFProc"
        _stop
        ;;

    version)
        echo @VERSION@
        exit 0
        ;;

    *)
        echo "Usage: $0 { start | stop }"
        exit 1
        ;;
esac
exit $?
EOT
}

# Redhat_init_script

sub Redhat_init_script
{
    return <<'EOT';
#! @SH@
#
@SERVICE_HEADER@
# description: Start the Process (Daemon) for the Software Test Automation Framework (STAF)
# processname: STAFProc
# owner: dl-nate-dev@netapp.com

# Source function library.
. @FUNCTIONS@

update()
{
    if [ -e @INSTALL_FROM@ ]; then
        diff @INSTALL_FROM@/install.properties @STAFDIR@/install.properties > /dev/null 2>&1
        if [ $? != 0 ]; then
            install
        fi
    fi
}

stafproc_exists()
{
    i=0
    REPEATS=60
    while [ $i -lt $REPEATS ]
        do
            sleep 1
            if [ -e @STAFDIR@/bin/STAFProc ]; then
                # true
                return 1
            fi
            i=`expr $i + 1`
    done
    echo @STAFDIR@/bin/STAFProc Does Not exist
    echo "@STAFDIR@/bin/STAFProc Does Not exist" >> /var/log/STAFProc.log
    # false
    return 0
}

install()
{
    if [ "@INSTALL_FROM@" != "@STAFDIR@" ]; then
        rm -rf @STAFDIR@
        cp -r -L @INSTALL_FROM@ @STAFDIR@
    fi
}

start()
{
    if [ -e @PERL@ ] && [ -e @START_STAF@ ]; then
        @PERL@ @START_STAF@ @STARTUP_ARGS@ > /dev/null 2>&1
    fi

    if [ @UPDATE@ == 1 ]; then
        update
    fi

    ulimit -c unlimited
    ulimit -v unlimited
    ulimit -l unlimited
    ulimit -u unlimited
    echo 999999 > /proc/sys/fs/file-max
    limit_ans=`su natedev -s /bin/sh -c "ulimit -n"`
    if [ $limit_ans -lt 999999 ]; then
        echo -e "\nnatedev - nofile 999999" >> /etc/security/limits.conf
    fi
    limit_ans=`su -s /bin/sh -c "ulimit -n"`
    if [ $limit_ans -lt 999999 ]; then
        echo -e "\nroot - nofile 999999" >> /etc/security/limits.conf
    fi
    ulimit -n 999999
    PATH=@PATH@
    export PATH
    LD_LIBRARY_PATH=@LD_LIBRARY_PATH@
    export LD_LIBRARY_PATH
    export STAF_TEMP_DIR=@STAF_TEMP_DIR@
    export STAF_DIR=@STAFDIR@
    export STAFCONVDIR=@STAFDIR@/codepage
    export STAF_DEBUG_RC_21=1
    status_of_proc
    if [ $? == 0 ]; then
        echo STAFProc is Already Running
        return 0
    fi

    rm -f /tmp/DataDir_-var-staf-data.tmp
    rm -f @STAF_TEMP_DIR@/DataDir_-var-staf-data.tmp
    rm -f /tmp/STAFIPC_STAF
    rm -f @STAF_TEMP_DIR@/STAFIPC_STAF
    rm -f /tmp/STAF.tmp
    rm -f @STAF_TEMP_DIR@/STAF.tmp

    rm -f /var/log/STAFProc_5.log
    mv -f /var/log/STAFProc_4.log /var/log/STAFProc_5.log > /dev/null 2>&1
    mv -f /var/log/STAFProc_3.log /var/log/STAFProc_4.log > /dev/null 2>&1
    mv -f /var/log/STAFProc_2.log /var/log/STAFProc_3.log > /dev/null 2>&1
    mv -f /var/log/STAFProc_1.log /var/log/STAFProc_2.log > /dev/null 2>&1
    mv -f /var/log/STAFProc.log   /var/log/STAFProc_1.log > /dev/null 2>&1

    stafproc_exists
    if [ $? == 0 ]; then
        failure
        RETVAL=1
        return $RETVAL
    fi

    cd /

    # The below command is to start mysql server (as 'natedev') for ntest to use.
    if [ -e @NATEDIR@/bin/start_stop_mysql_server.pl ] && [ @MYSQLD@ -eq 1 ]; then
        su -m -s /bin/sh natedev -c '@PERL@ @NATEDIR@/bin/start_stop_mysql_server.pl'
    fi

    echo -n "Starting STAFProc: "
    # The behavior of 'su' with '-m'(Preserve the current environment) is
    # not consistent across all Unix operating systems. For instance, it
    # is not preserving PATH env variable in latest version of Ubuntu.
    # So, we are specifying all the required PATH and other env variables
    # explicitly for STAFProc process (BURT 656364).
    nohup @SPECIAL_USER@ "PATH=@PATH@ LD_LIBRARY_PATH=@LD_LIBRARY_PATH@ \
        STAF_TEMP_DIR=@STAF_TEMP_DIR@ STAF_DIR=@STAFDIR@ STAFCONVDIR=@STAFDIR@/codepage \
        @STAFDIR@/bin/STAFProc @CFG_FILE@" > /var/log/STAFProc.log 2>&1 &

    # If STAFProc cannot be run, a process probably will still
    # have been created, but will exit soon after.  Test for that
    # as best we can.
    i=0
    REPEATS=30
    while [ $i -lt $REPEATS ]
        do
            sleep 1
            status_of_proc
            if [ $? == 0 ]; then
                sleep 2
                i=$REPEATS
            fi
            i=`expr $i + 1`
    done
    PID=`pidof @STAFDIR@/bin/STAFProc`

    if [ ! -e /proc/$PID ]; then
            failure
            RETVAL=1
            return $RETVAL
    fi
    echo $PID > /var/run/STAFProc.pid
    if [ ! -e /var/lock/subsys ]; then
        mkdir -p /var/lock/subsys
    fi
    touch /var/lock/subsys/STAFProc
    # Because it has so many children that may use a lot of
    # memory, STAFProc itself may be a good Out Of Memory kill
    # target, but we would prefer other processes (such as those
    # children individually) to be chosen instead.

    [ -e /proc/$PID/oom_adj ] && echo -17 > /proc/$PID/oom_adj
    [ -e /proc/$PID/oomadj  ] && echo -17 > /proc/$PID/oomadj

    if [ -e /tmp/DataDir_-var-staf-data.tmp ]; then
        if [ ! -e @STAF_TEMP_DIR@/DataDir_-var-staf-data.tmp ]; then
            ln -s /tmp/DataDir_-var-staf-data.tmp @STAF_TEMP_DIR@/DataDir_-var-staf-data.tmp
        fi
    fi
    if [ -e /tmp/STAFIPC_STAF ]; then
        if [ ! -e @STAF_TEMP_DIR@/STAFIPC_STAF ]; then
            ln -s /tmp/STAFIPC_STAF @STAF_TEMP_DIR@/STAFIPC_STAF
        fi
    fi
    if [ -e /tmp/STAF.tmp ]; then
        if [ ! -e @STAF_TEMP_DIR@/STAF.tmp ]; then
            ln -s /tmp/STAF.tmp @STAF_TEMP_DIR@/STAF.tmp
        fi
    fi
    if [ -e @STAF_TEMP_DIR@/DataDir_-var-staf-data.tmp ]; then
        if [ ! -e /tmp/DataDir_-var-staf-data.tmp ]; then
            ln -s @STAF_TEMP_DIR@/DataDir_-var-staf-data.tmp /tmp/DataDir_-var-staf-data.tmp
        fi
    fi
    if [ -e @STAF_TEMP_DIR@/STAFIPC_STAF ]; then
        if [ ! -e /tmp/STAFIPC_STAF ]; then
            ln -s @STAF_TEMP_DIR@/STAFIPC_STAF /tmp/STAFIPC_STAF
        fi
    fi
    if [ -e @STAF_TEMP_DIR@/STAF.tmp ]; then
        if [ ! -e /tmp/STAF.tmp ]; then
            ln -s @STAF_TEMP_DIR@/STAF.tmp /tmp/STAF.tmp
        fi
    fi

    is_stafproc_ready
    if [ $? == 0 ]; then
            failure
            RETVAL=1
            return $RETVAL
    fi

    @SUCCESS@
    echo
    return $RETVAL
}

is_stafproc_ready()
{
    i=0
    # 4 minutes
    REPEATS=240
    if [ $? != 0 ]; then
        echo
        echo -n "Please wait.  This may take up to 4 minutes"
    fi
    while [ $i -lt $REPEATS ]
        do
            @STAFDIR@/bin/STAF local REAPER VERSION > /dev/null 2>&1
            if [ $? == 0 ]; then
                # true
                return 1
            fi
            sleep 1
            i=`expr $i + 1`
    done
    echo "It appears that a Service has not started"
    echo `cat /var/log/STAFProc.log`
    # false
    return 0
}

status_of_proc()
{
    @PROC_TEST@
    return $?
}

stop()
{
    # Stop daemons.
    echo -n "Shutting down STAFProc: "
    @KILL@
    RETVAL=$?

    i=0
    REPEATS=30
    while [ $i -lt $REPEATS ]
        do
            sleep 1
            status_of_proc
            if [ $? != 0 ]; then
                sleep 2
                i=$REPEATS
                @SUCCESS@
            fi
            i=`expr $i + 1`
    done

    rm -f /tmp/DataDir_-var-staf-data.tmp
    rm -f @STAF_TEMP_DIR@/DataDir_-var-staf-data.tmp
    rm -f /tmp/STAFIPC_STAF
    rm -f @STAF_TEMP_DIR@/STAFIPC_STAF
    rm -f /tmp/STAF.tmp
    rm -f @STAF_TEMP_DIR@/STAF.tmp
    echo
    [ $RETVAL -eq 0 ] && rm -f /var/lock/subsys/STAFProc
    return $RETVAL
}

# See how we were called.
case "$1" in
    start)
        start
        RETVAL=$?
        ;;
    stop)
        stop
        RETVAL=$?
        ;;
    reload|restart)
        stop
        sleep 5
        start
        RETVAL=$?
        ;;
    condrestart)
        if [ -f /var/lock/subsys/STAFProc ]; then
            stop
            sleep 5
            start
            RETVAL=$?
        fi
        ;;
    status)
        status STAFProc
        RETVAL=$?
        ;;
    test)
        status STAFProc
        RETVAL=$?
        ;;
    version)
        echo @VERSION@
        exit 0
        ;;
    *)
        echo "Usage: $0 {start|stop|restart|condrestart|status}"
        exit 1
esac

exit $RETVAL

EOT
}

sub smf_init_script
{
    return <<'EOT';
#! @SH@

# owner: dl-nate-dev@netapp.com
# Source function library.
. @FUNCTIONS@

update()
{
    if [ -e @INSTALL_FROM@ ]; then
        diff @INSTALL_FROM@/install.properties @STAFDIR@/install.properties > /dev/null 2>&1
        if [ $? != 0 ]; then
            install
        fi
    fi
}

install()
{
    if [ "@INSTALL_FROM@" != "@STAFDIR@" ]; then
        rm -rf @STAFDIR@
        cp -r -L @INSTALL_FROM@ @STAFDIR@
    fi
}

stop ()
{
    /usr/bin/pkill -x -u 0 STAFProc
    RVAL=$?
    i=0
    REPEATS=30
    while [ $i -lt $REPEATS ]
        do
            sleep 1
            status > /dev/null
            if [ $? != 0 ]; then
                sleep 2
                i=$REPEATS
            fi
            i=`expr $i + 1`
    done

    rm -f /tmp/DataDir_-var-staf-data.tmp
    rm -f @STAF_TEMP_DIR@/DataDir_-var-staf-data.tmp
    rm -f /tmp/STAFIPC_STAF
    rm -f @STAF_TEMP_DIR@/STAFIPC_STAF
    rm -f /tmp/STAF.tmp
    rm -f @STAF_TEMP_DIR/@STAF.tmp
    return $RVAL;
}

stafproc_exists()
{
    i=0
    REPEATS=60
    while [ $i -lt $REPEATS ]
        do
            sleep 1
            if [ -e @STAFDIR@/bin/STAFProc ]; then
                return 1
            fi
            i=`expr $i + 1`
    done
    echo @STAFDIR@/bin/STAFProc Does Not exist
    return 0
}

condrestart()
{
    if [ -f /var/lock/subsys/STAFProc ]; then
        stop
        return start
    fi
}

restsart()
{
    stop
    return start
}

status ()
{
    @PROC_TEST@
    return $?
}

start()
{
    if [ -e @PERL@ ] && [ -e @START_STAF@ ]; then
        @PERL@ @START_STAF@ @STARTUP_ARGS@ > /dev/null 2>&1
    fi

    if [ @UPDATE@ == 1 ]; then
        update
    fi

    status > /dev/null
    if [ $? == 0 ]; then
        echo STAFProc is Already Running
        return 0
    fi
    stafproc_exists
    if [ $? == 0 ]; then
        return 1;
    fi
    ulimit -c unlimited
    ulimit -v unlimited
    ulimit -n unlimited
    ulimit -s unlimited
    echo "Starting STAFProc: "
    RETVAL=$SMF_EXIT_OK
    PATH=@PATH@
    export PATH
    LD_LIBRARY_PATH=@LD_LIBRARY_PATH@
    export LD_LIBRARY_PATH
    export STAF_TEMP_DIR=@STAF_TEMP_DIR@
    export STAF_DIR=@STAFDIR@
    export STAFCONVDIR=@STAFDIR@/codepage
    export STAF_DEBUG_RC_21=1
    rm -f /tmp/DataDir_-var-staf-data.tmp
    rm -f @STAF_TEMP_DIR@/DataDir_-var-staf-data.tmp
    rm -f /tmp/STAFIPC_STAF
    rm -f @STAF_TEMP_DIR@/STAFIPC_STAF
    rm -f /tmp/STAF.tmp
    rm -f @STAF_TEMP_DIR@/STAF.tmp

    cd /
    @STAFDIR@/bin/STAFProc @CFG_FILE@ &

    i=0
    REPEATS=30
    while [ $i -lt $REPEATS ]
        do
            sleep 1
            status > /dev/null
            if [ $? == 0 ]; then
                sleep 2
                i=$REPEATS
            fi
            i=`expr $i + 1`
    done

    if [ -e /tmp/DataDir_-var-staf-data.tmp ] && [ ! -e @STAF_TEMP_DIR@/DataDir_-var-staf-data.tmp ]
        then
            ln -s /tmp/DataDir_-var-staf-data.tmp @STAF_TEMP_DIR@/DataDir_-var-staf-data.tmp
    fi

    if [ -e /tmp/STAFIPC_STAF ] && [ ! -e @STAF_TEMP_DIR@/STAFIPC_STAF ]
        then
            ln -s /tmp/STAFIPC_STAF @STAF_TEMP_DIR@/STAFIPC_STAF
    fi

    if [ -e /tmp/STAF.tmp ] && [ ! -e @STAF_TEMP_DIR@/STAF.tmp ]
        then
            ln -s /tmp/STAF.tmp @STAF_TEMP_DIR@/STAF.tmp
    fi

    if [ -e @STAF_TEMP_DIR@/DataDir_-var-staf-data.tmp ] && [ ! -e /tmp/DataDir_-var-staf-data.tmp ]
        then
            ln -s @STAF_TEMP_DIR@/DataDir_-var-staf-data.tmp /tmp/DataDir_-var-staf-data.tmp
    fi

    if [ -e @STAF_TEMP_DIR@/STAFIPC_STAF ] && [ ! -e /tmp/STAFIPC_STAF ]
        then
            ln -s @STAF_TEMP_DIR@/STAFIPC_STAF /tmp/STAFIPC_STAF
    fi

    if [ -e @STAF_TEMP_DIR@/STAF.tmp ] && [ ! -e /tmp/STAF.tmp ]
        then
            ln -s @STAF_TEMP_DIR@/STAF.tmp /tmp/STAF.tmp
    fi

    is_stafproc_ready
    if [ $? == 0 ]; then
        return 1;
    fi
}

is_stafproc_ready()
{
    i=0
    # 4 minutes
    REPEATS=240
    if [ $? != 0 ]; then
        echo
        echo "Please wait.  This may take up to 4 minutes"
    fi
    while [ $i -lt $REPEATS ]
        do
            @STAFDIR@/bin/STAF local REAPER VERSION > /dev/null 2>&1
            if [ $? == 0 ]; then
                # true
                return 1
            fi
            sleep 1
            i=`expr $i + 1`
    done
    echo "It appears that a Service has not started"
    echo `cat /var/log/STAFProc.log`
    # false
    return 0
}

# See how we were called.
case "$1" in
    restart)
        restsart
        RETVAL=$?
        ;;
    condrestart)
        condrestart
        RETVAL=$?
        ;;
    stop)
        stop
        RETVAL=$?
        ;;
    start)
        start
        RETVAL=$SMF_EXIT_OK
        ;;
    refresh)
        status
        RETVAL=$?
        ;;
    version)
        echo @VERSION@
        exit 0
        ;;
    status)
        status
        RETVAL=$?
        ;;
    *)
        echo "Usage: $0 {start|stop|restart|condrestart|status}"
        exit 1

esac

exit $RETVAL

EOT
}
